<?php
namespace Controller;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;

class HierarchyType extends Crud{
    protected $dataSourceName = 'hierarchyType';

    public function find (DTO\Request\Filter $request, DTO\Response $response) {
        parent::find($request, $response);
        $dataSet = $response->getDataSets()[0];
        $hierarchyTypes = $dataSet->getRows();
        $strucuresTypes = $this->getStrucutresTypes($hierarchyTypes)->getRows();
        $indexedStructureTypes = $this->indexByMongoId($strucuresTypes);
        $hierarchyTypes = $this->relateHierarchyWithStructureType($hierarchyTypes, $indexedStructureTypes);
        $indexedHierarchyTypes = $this->indexByMongoId($hierarchyTypes);
        $hierarchyTypes = $this->relateHierarchyWithHierarchyParent($hierarchyTypes, $indexedHierarchyTypes);
        $newDataSet = new DataSet($this->dataSourceName, $hierarchyTypes);
        $response->setDataSets(array($newDataSet));
    }

    protected function getStrucutresTypes ($hierarchyTypes) {
        $usedIds = $this->getStructureIdsUsed($hierarchyTypes);
        $numberOfExpectedLines = count($usedIds);
        $filterCriteria = new FilterCriteria('structureType', array(), null, $numberOfExpectedLines);
        $filterCriteria->addCondition('_id', FilterCriteria::IN, $usedIds);
        return $this->mongoManager->findBy($filterCriteria);
    }

    protected function getStructureIdsUsed ($hierarchyTypes) {
        //For not repeat Ids used more than one time
        $structuresIdsIndexedById = $this->getDistinctStructureIds($hierarchyTypes);
        //Build distinct ids in a index array
        $distictStruturesIds = array();
        foreach($structuresIdsIndexedById as $structureId) {
            $distictStruturesIds[] = $structureId;
        }
        return $distictStruturesIds;
    }

    protected function getDistinctStructureIds($hierarchyTypes) {
        $structuresIdsIndexedById = array();
        foreach($hierarchyTypes as $hierarchyType) {
            $parentTypeId = $hierarchyType['typeParent'];
            $structuresIdsIndexedById[$parentTypeId] = $parentTypeId;
            $childTypeId = $hierarchyType['typeChild'];
            $structuresIdsIndexedById[$childTypeId] = $childTypeId;
        }
        return $structuresIdsIndexedById;
    }

    protected function indexByMongoId($rows) {
        $newRows = array();
        foreach($rows as $row) {
            $newRows[$row['_id']] = $row;
        }
        return $newRows;
    }

    protected function relateHierarchyWithStructureType($hierarchyTypes, $indexedByIdStructureTypes) {
        foreach($hierarchyTypes as &$hierarchyType) {
            $hierarchyType['typeParentName'] = $indexedByIdStructureTypes[$hierarchyType['typeParent']]['name'];
            $hierarchyType['structureTypeParent'] =  $indexedByIdStructureTypes[$hierarchyType['typeParent']];
            $hierarchyType['typeChildName'] = $indexedByIdStructureTypes[$hierarchyType['typeChild']]['name'];
            $hierarchyType['structureTypeChild'] =  $indexedByIdStructureTypes[$hierarchyType['typeChild']];
        }
        return $hierarchyTypes;
    }

    protected function relateHierarchyWithHierarchyParent($hierarchyTypes, $indexedByIdHierarchyTypes) {
        foreach($hierarchyTypes as &$hierarchyType) {
            if (isset($hierarchyType['parent'])) {
                $hierarchyType['parentName'] = $indexedByIdHierarchyTypes[$hierarchyType['parent']]['name'];
            }
        }
        return $hierarchyTypes;
    }
}